#include <math.h>
#include <stdio.h>
#include <string.h>                                                
#include <stdlib.h>
#include <time.h>
//                  James A. Green, June 17,2002, Greenwood Research
//                  james_a_green_iii@yahoo.com, http://grnwd.tripod.com.
//
//  starz6ab.cpp - Manage a file of the abundances of the elements.
//
//  elements() - if ioenable is 1,
//                 Input abundances in parts by weight from z6/abundan.txt,
//                 compute fractional parts, display, edit, write back to
//	           z6/abundan.txt, 
//               else
//                 use internal record of Aller 1961 default with
//                 purely run-time modification of internal tables.
//
//  GLOBAL VARIABLES
//
int ioenable = 1;   // COMPILE-TIME DISK IO OPTION
//                    -> 1 for STARMODL, -> 0 for STARMAN, to avoid extra files.
//                    -> 1 for ELEMENTS.

//           Aller 1961 default.
//   Working buffer showing default in parts by weight, modified by z6/abundan.txt contents.
//		                H      He
double xb[30] = { 0.00000000, 1000.0, 640.0, 0.00000000, 0.00000000, 0.00000000,
		  4.80000, 1.5, 14.0, 0.02, 10.0,
		  0.0500, 0.60, 0.05, 0.9, 0.00000000,
		  0.7, 0.07, 0.3, 0.00000000, 0.08000,
		 0.00000000, 0.00000000, 0.00000000, 0.000000000, 0.000000000,
		 0.20, 0.00000000, 0.06, 0.01 };
//
//   Aller 1961 default in parts by weight, Clayton, p.85, for
//   restoring default in parts by weight on disk. Invarient regardless of 
//   z6/abundan.txt contents, also used to restore xb[N], above, and x[N] fractionals.
//
double xn[30] = { 0.00000000, 1000.0, 640.0, 0.00000000, 0.00000000, 0.00000000,
		  4.80000, 1.5, 14.0, 0.02, 10.0,
		  0.0500, 0.60, 0.05, 0.9, 0.00000000,
		  0.7, 0.07, 0.3, 0.00000000, 0.08000,
		  0.00000000, 0.00000000, 0.00000000, 0.000000000, 0.000000000,
		  0.20, 0.00000000, 0.06, 0.01 };
//
//   Element tags.
//
char el2[30] = "T HLB     NNMAS  CA CST CMFCN>"; // Tag the elements, using
char el3[30] = "UHeieBCNOFeagliPSlrKaciVrneoiN"; // a vertical acrostic.
char el4[30] = "                             i";

void elementv(void);

//
//           Solar Abundance X(Z) by weight, Pop.I model abundances, Clayton, p.85.
//                  Fractional abundances by weight, after Aller 1961.
//
//             Working buffer for actual fractional parts used in STARMODL.
//
// EXTERNAL VARIABLES
//
extern double x[30];
extern double d;
extern char value[50];
extern int chemdisplay; // 0 -> Do not display X,Y,Z & CNO chemistry conclusions.
//                         1 -> Display X,Y,Z & CNO chemistry conclusions.
extern double chemx[1];
extern double chemy[1];
extern double chemz[1];
extern double chemmu[1];
extern double real_xcno[1];  // x[6] plus x[7] (Carbon plus Nitrogen),
//                              0.011210 for ZAMS Sirius, x3 C, x3 N if Aller 1961 used.
extern double xcno;          // was using 0.008909.

void elements(void)
{
	int status;
        FILE *fp5;
	char rdbuf[80];
	char strgbuf[20];
	int jj;
	int select, kik;
	double claywt;
	void newclayx(void);
	double abunsum; // Sum of abundances by weight.
	void restorex(void);

        if(ioenable == 1){ // disk io of table may be suspended.
	    fp5 = fopen("z6/abundan.txt", "r"); // Read in abundances in parts by wt.
//                                       abundan2.txt has ZAMS abundances for Sirius
//                                       when the Schwarzschild CNO cycle formulae are used.
	    kik = 0;
	    fscanf(fp5,"%s",rdbuf);     //  Read the first title string, no spaces.
	    while(kik < 30){            //  Read and convert the numeric data -> xb[30],
	      fscanf(fp5,"%s",rdbuf);   //  the list of abundances in parts by weight.
	      fscanf(fp5,"%s",rdbuf);   // 1st read is element tag, 2nd is value.
	      d = atof(rdbuf);
	      xb[kik] = d;
	      kik++;
	      fscanf(fp5,"%s",rdbuf);  // 1st read is element tag, 2nd is value.
	      fscanf(fp5,"%s",rdbuf);
	      d = atof(rdbuf);
	      xb[kik] = d;
	      kik++;
	    }  
	    status = fclose(fp5);   // Close file of abundances in parts by wt.
	}


	kik = 0;
	claywt = 0.0;  // Precompute Claytons p.85 sum of abundances by weight for display.
	while(kik < 30){                // xb[kik] abundances in part by weight.
	   claywt = claywt + xb[kik];   // We may use claywt later to convert back to parts.
	   kik++;
	}

	kik = 0;       // Find the corresponding fractional abundances by weight.
	while(kik < 30){        // x[kik] fractional abundances by weight.
	   x[kik] =  xb[kik]/claywt;
	   kik++;
	}
	
	abunsum = 0.0;
	kik = 0;       // Precompute the sum of abundances by weight for display.
	while(kik < 30){        //  Sum of fractional abundances should be unity.
	   abunsum = abunsum + x[kik];
	   kik++;
	}
	select = 1;
	while(select == 1){
	   printf("ELEMENT ABUNDANCES: X(H)=0.6, Y(He)=0.38, Z(hvy)=0.02, typical.\n");
	   printf("Abundances in parts by weight are kept in z6/abundan.txt.\n");
// 	   printf("Used for electrostatic screening of nuclear reactions, CNO, opacity,\n");
// 	   printf("nuclear reactions in general, ect.  TU is the NULL element.\n");
// 	   printf("The user is at liberty to specify X(H),Y(He),Z(hvy) independantly, with\n");
// 	   printf("heavy element abundances Z(hvy) shaped like the Aller 1961 default.\n");
	   printf("Sum of fractional abundances by_Weight for Z=1 thru 29 = %1.8f.\n", abunsum);
	   printf(" Z Element by_Weight Parts_by_Wt | Z Element by_Weight Parts_by_Wt.");
	   kik = 0;
	   while(kik < 29){
	      sprintf(strgbuf,"%f ",xb[kik]);
	      printf("\n%2d %c%c%c -> %1.8f, ",kik,el2[kik],el3[kik],el4[kik],x[kik]);
	      for(jj = 0; jj < 8; jj++)
		 printf("%c", strgbuf[jj]);
	      kik++;
	      sprintf(strgbuf,"%f ",xb[kik]);
	      printf("    %2d %c%c%c -> %1.8f, ",kik,el2[kik],el3[kik],el4[kik],x[kik]);
	      for(jj = 0; jj < 8; jj++)
		 printf("%c", strgbuf[jj]);
	      kik++;
	   }
	   printf("\n");
	   select = 1;
	   if(select == 1){
	       chemx[0]  = x[1];  chemy[0] = x[2], chemz[0] = 1.0 - chemx[0] - chemy[0];
	       chemmu[0] = 1.0/((2.0*chemx[0]) + (0.75*chemy[0]) + (0.5*chemz[0]));
//                           Carbon Nitrogen
	       real_xcno[0] = x[6] + x[7];
	       printf("X(H) = %f, Y(He) = %f, Z(hvy) = %f.\n", chemx[0], chemy[0], chemz[0]);
	       printf("real_CN = x[6] + x[7] = %f, from table x[Z], above.\n", real_xcno[0]);
	       printf("Schwarz_CN = Z/3 = %f, M.Schwarzschild toy model for X_CN.\n",
		 chemz[0]/3.0);
               printf("X_CN = %f. Model used in abundance options 1-5.\n",xcno);
	       printf("ENTER the Z-index of the element to be modified,\n");	       
	       printf("0 to restore the Pop I Aller 1961 default, or 30 to continue: ");
	       scanf("%s",value);       // Entry procedure more immune to
	       d = atof(value);         // invalid inputs than simple scanf
	       kik = d;
	       if(kik > 0 && kik < 30){		  
		      printf("%2d %c%c -> %1.8f,  %f ",kik,el2[kik],el3[kik],x[kik],xb[kik]);
		      printf("ENTER new Parts_by_Wt value : ");
	              scanf("%s",value);       // Entry procedure more immune to
		      d = atof(value);         // invalid inputs than simple scanf
		      xb[kik] = d;
		      newclayx();              // recompute entire fractional abundances x[].
		      printf("%2d %c%c -> %1.8f, %f -> ",kik,el2[kik],el3[kik],x[kik],xb[kik]);
		      abunsum = 0.0;
	              kik = 0;
		      while(kik < 30){   // recompute the sum of abundances.
		         abunsum = abunsum + x[kik];
	                 kik++;
		      }
		      printf("New sum of abundances by weight = %1.8f.\n", abunsum);
	       }	   
	       else if(kik == 30 || kik > 30)
		 select = 0; // if exit signal, return to main menu.
	       else if(kik == 0){
		 select = 1; // if kik out of range, replay,
		 restorex(); // and restore default abundances in parts by weight.
	       }
	   }
	}
//
//         Now write the modified list of abundances in parts by wt to disk.
//
        if(ioenable == 1){ // disk io of modified table may be suspended.
	    fp5 = fopen("z6/abundan.txt", "w"); //Write out the abundances in parts by wt.

	    fprintf(fp5,"Abundances_in_parts_by_weight_TU_is_NULL\n");
	    kik = 0;
            while(kik < 30){
	       fprintf(fp5,"%c%c%c   %f\n",el2[kik],el3[kik],el4[kik],xb[kik]);
	       kik++;
	    }
	    fprintf(fp5,"Default is Aller 1961, Clayton pg.85.\n");

	    status = fclose(fp5);

	    printf("Wrote abundances in parts by weight to z6/abundan.txt.\n");
	}
}
//
//  newclayx - Compute the new fractional abundances x[kak] from the
//             newly-modified list of abundances in parts by weight xb[kak].
//
void newclayx(void)
{
	double nclaywt;
	int kak;

        kak = 0;
	nclaywt = 0.0; // Find edited sum of abundances in parts by weight for display.
	while(kak < 30){ 
	   nclaywt = nclaywt + xb[kak];   // Use nclaywt later to convert to fractional form.
	   kak++;
	}
	kak = 0;       // Find the new fractional abundances from the new parts by wt.
	while(kak < 30){ 
	   x[kak] = xb[kak]/nclaywt;   // Use nclaywt to convert to fractional form.
	   kak++;
	}
}
//
// restorex() -  Restore the original Aller 1961
//               list of abundances in parts by weight to disk.
//
void restorex(void)
{
        int status;
        FILE *fp5;	
	int kik;
	double claywt;
	double abunsum;

        if(ioenable == 1){
	   fp5 = fopen("z6/abundan.txt", "w"); //Write out the default abundances, parts by wt.

	   fprintf(fp5,"Abundances_in_parts_by_weight_TU_is_NULL\n");
	   kik = 0;
           while(kik < 30){
	      fprintf(fp5,"%c%c%c   %f\n",el2[kik],el3[kik],el4[kik],xn[kik]);
	      kik++;
	   }
	   fprintf(fp5,"Default is Aller 1961, Clayton pg.85.\n");

	   status = fclose(fp5);
     
	   printf("Wrote abundances in parts by weight to z6/abundan.txt.\n");
	}

	kik = 0;
	claywt = 0.0;  // Precompute Claytons p.85 sum of abundances by weight for display.
	while(kik < 30){
	   xb[kik] = xn[kik];           // Restore internal abundances by wt. to default.
	   claywt = claywt + xb[kik];   // We may use claywt later to convert back to parts.
	   kik++;
	}

	kik = 0;       // Restore the corresponding fractional abundances by weight.
	while(kik < 30){ 
	   x[kik] =  xb[kik]/claywt;
	   kik++;
	}
}
//
//   elementv() - Inialize internal tables of the elements from disk.
//
void elementv(void)
{
	int status;
        FILE *fp5;
	char rdbuf[80];
	int kik;
	double claywt;

        
	fp5 = fopen("z6/abundan.txt", "r"); // Read in abundances in parts by wt.
//                                       abundan2.txt has ZAMS abundances for Sirius
//                                       when the Schwarzschild CNO cycle formulae are used.
	kik = 0;
        fscanf(fp5,"%s",rdbuf);     //  Read the first title string, no spaces.
	while(kik < 30){            //  Read and convert the numeric data -> xb[30],
	      fscanf(fp5,"%s",rdbuf);   //  the list of abundances in parts by weight.
	      fscanf(fp5,"%s",rdbuf);   // 1st read is element tag, 2nd is value.
	      d = atof(rdbuf);
	      xb[kik] = d;
	      kik++;
	      fscanf(fp5,"%s",rdbuf);  // 1st read is element tag, 2nd is value.
	      fscanf(fp5,"%s",rdbuf);
	      d = atof(rdbuf);
	      xb[kik] = d;
	      kik++;
	}  
	status = fclose(fp5);   // Close file of abundances in parts by wt.


	kik = 0;
	claywt = 0.0;  // Precompute Claytons p.85 sum of abundances by weight for display.
	while(kik < 30){                // xb[kik] abundances in part by weight.
	   claywt = claywt + xb[kik];   // We may use claywt later to convert back to parts.
	   kik++;
	}

	kik = 0;       // Find the corresponding fractional abundances by weight.
	while(kik < 30){        // x[kik] fractional abundances by weight.
	   x[kik] =  xb[kik]/claywt;
	   kik++;
	}
}

//   James A. Green, June 17,2002, Greenwood Research
//   james_a_green_iii@yahoo.com, http://grnwd.tripod.com.