#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
//
//                                      Programmer: James A. Green
//  starz4au.cpp - auxluminosities().               May 2002
//                 Compute auxiliary
//                 luminosities for the stellar wind
//                 and the neutrino luminosity.
//
//    - PROJECT STARMAN -  Programmer: James A. Green.
//	   starman.prj is the project file
//	       producing starman.exe from
//		     starz7.cpp - main module.
//		      starz7ag.cpp - Stellar ages and evolution, perturbations.
//		      starz7au.cpp - Neutrino luminosity & stellar wind.
//		      starz7fu.cpp - basic functions.
//		      starz7bo.cpp - bolometric corrections.
//		      starz7op.cpp - supplemental ops.
//
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .
//
//
extern double nthpower(double v, int i); // ith power of v
extern double bcmratiobest[1];           // beststarmass/solarmass
extern double gnewton;           // Newtons 6.67e-11 for grav.
extern double xmstar[1];         // Mass of the star in Kg.
extern double xrstar[1];         // Radius of the star in meters.
extern FILE *fp;                 // File pointer
extern int status;               // file status
extern double lsun;              // Luminosity of the Sun in Watts.
extern double chemx[1];          // X(H)
extern double chemz[1];          // Z(hvy)
extern double nscchiuluminosity[1];  // Screened, then normalized Chiu pp-fusion chain L(R).
//
void auxluminosities(void)
{
double lumwind;	 // L_stellar_wind(R) >= (GM/R)dM/dt Watts. Show also L_stellar_wind/L_sun.
double massloss; // dM/dt = 0.4e-13(M/M_sun)**4 solar masses per year, in kg per sec.
//                        = 2.521166e9(M/M_sun)**4 kg per second.
double massloss2;   // dM/dt = 0.4e-13(M/M_sun)**4 solar masses per year

//  A J.A.Green best fit formula based on Clayton data and Bowers & Deeming estimates.
//
//      Simplisitic neutrino luminosity measures - to be improved -
//
double neutrinolum; // neutrino luminosity 2% to 3% of luminosity, according to many scholars,
//		       for a primarily p-p fusion star. Given in Watts.
//
//  neutrinolum = neutrinosH*(4*pi*R**2)*E_neu(H)          E_neu(H)_avg =  0.26 MeV
//              + neutrinosppe*(4*pi*R**2)*E_neu(ppe)      E_neu(ppe)_avg = 1.44 MeV
//              + neutrinosBe*(4*pi*R**2)*E_neu(Be)        E_neu(Be)_avg = 0.86 MeV
//              + neutrinosB*(4*pi*R**2)*E_neu(B)          E_neu(B)_avg =  7.2 MeV
//              + neutrinosN*(4*pi*R**2)*E_neu(N)          E_neu(N)_avg =  0.71 MeV
//              + neutrinosO*(4*pi*R**2)*E_neu(O)          E_neu(O)_avg =  1.0 MeV
//
//  See Bowers & Deeming, ASTROPHYSICS I: The Stars, pg.242 for ppe neutrinos, full table.
//     Here  R = 1 AU = 1.496e11  meters, and 1 MeV = 1.6025e-13 Joules.
//
//  Population I NEUTRINO LUMINOSITY ESTIMATES via Chiu, STELLAR PHYSICS, pg.485.
//   Like Clayton, Chiu uses Aller Abundances from 1961.
//      X_Aller1961(H) = 0.6 , Y_Aller1961(He) = 0.38, Z_Aller1961(hvy) = 0.02.
//
double neutrinos;    // neutrinos per sec per square meter at orbital radius of 1 AU.
double neutrinosH;   // = 5.91e14*((X/X_All61)**2)*(L/L_sun) neu/sec-m**2 for H(p,beta+,nu)D2
double neutrinosppe; // = (.0155/6.063)*5.91e14*((X/X_All61)**2)*(L/L_sun) neu/sec-m**2
double neutrinosBe;  // = 0.69e14*(Z/Z_All61)*(L/L_sun)  neutrinos/sec-m**2 for Be7(e-,nu)Li7
double neutrinosB;   // = 0.95e11*(Z/Z_All61)*(L/L_sun) neutrinos/sec-m**2 for B8(beta+,nu)Be8
double neutrinosN;   // = 0.6e13*(Z/Z_All61)*(L/L_sun) neutrinos/sec-m**2 for N13(beta+,nu)C13
double neutrinosO;   // = 0.51e13*(Z/Z_All61)*(L/L_sun) neutrinos/sec-m**2 for O15(beta+,nu)N15
//
//  Just above the surface of the Sun,
//  [L_nuclear(R) - L_neutrino(R) - L_solarwind(R)]/L_sun = L_radiative(R)/L_sun = 1.
//
double lumratio;
double ehavg = 0.26; // 0.26 MeV , avg energy of neutrinosH,
double eppeavg = 1.44; // 1.44 MeV , avg energy of neutrinosppe,
double ebeavg = 0.86; // 0.86 MeV , avg energy of neutrinosBe,
double ebavg = 0.71; // 7.2 MeV , avg energy of neutrinosB,
double enavg = 0.86; // 0.86 MeV , avg energy of neutrinosBe,
double eoavg = 0.86; // 1.0 MeV , avg energy of neutrinosO,
double lum;          // nuclear luminosity
double au = 1.496e11; // 1 astronomical unit is 1.496e11 meters.
double mev = 1.6025e-13; // 1 MeV is 1.6025e-13 Joules
double surface;

	fprintf(fp,"\nAUX LUMINOSITIES - Estimates\n\n");

	massloss = (2.52166e9)*nthpower(bcmratiobest[0],4);  // in kg per sec
	massloss2 = (0.4e-13)*nthpower(bcmratiobest[0],4);  // in solar masses per year
	lumwind = (gnewton*xmstar[0]/xrstar[0])*massloss;
	fprintf(fp,"L_stellar_wind(R) = %e Joules/sec\n",lumwind);
	lumratio = lumwind/lsun;
	fprintf(fp,"L_stellar_wind(R)/L_sun = %e -> %f percent\n",lumratio, lumratio*100.0);
	fprintf(fp,"  Mass loss = %e kg/sec = %e solar masses/year\n",massloss, massloss2);
	lum = nscchiuluminosity[0];  // normed, screened Chiu luminosity in solar luminosities
	neutrinosH = 5.91e14*((chemx[0]/0.6)*(chemx[0]/0.6))*(lum); // for H(p,beta+,nu)D2
	neutrinosppe = (.0155/6.063)*5.91e14*((chemx[0]/0.6)*(chemx[0]/0.6))*(lum); // neu/sec-m**2
	neutrinosBe = 0.69e14*(chemz[0]/0.02)*(lum); // neutrinos/sec-m**2 for Be7(e-,nu)Li7
	neutrinosB = 0.95e11*(chemz[0]/0.02)*(lum); // neutrinos/sec-m**2 for B8(beta+,nu)Be8
	neutrinosN = 0.6e13*(chemz[0]/0.02)*(lum); // neutrinos/sec-m**2 for N13(beta+,nu)C13
	neutrinosO = 0.51e13*(chemz[0]/0.02)*(lum); // neutrinos/sec-m**2 for O15(beta+,nu)N15

	neutrinos = neutrinosH + neutrinosppe + neutrinosBe + neutrinosB + neutrinosN;
	neutrinos = neutrinos + neutrinosO;

	fprintf(fp,"neutrinos = %e neutrinos/sec per m**2 at 1 AU.\n",neutrinos);
	fprintf(fp,"neutrinosH = %e H(p,beta+,nu)D2 neutrinos/sec per m**2 at 1 AU.\n",neutrinosH);
	fprintf(fp,"neutrinosppe = %e ppe neutrinos/sec-m**2 at 1 AU.\n",neutrinosppe);
	fprintf(fp,"neutrinosBe = %e Be7(e-,nu)Li7 neutrinos/sec-m**2 at 1 AU.\n",neutrinosBe);
	fprintf(fp,"neutrinosB = %e B8(beta+,nu)Be8 neutrinos/sec-m**2 at 1 AU.\n",neutrinosB);
	fprintf(fp,"neutrinosN = %e N13(beta+,nu)C13 neutrinos/sec-m**2 at 1 AU.\n",neutrinosN);
	fprintf(fp,"neutrinosO = %e O15(beta+,nu)N15 neutrinos/sec-m**2 at 1 AU.\n",neutrinosO);


	surface = 4.0*3.14159*au*au;
	neutrinolum = neutrinosH*surface*ehavg*mev;
	neutrinolum =  neutrinolum + neutrinosppe*surface*eppeavg*mev;
	neutrinolum =  neutrinolum + neutrinosppe*surface*ebeavg*mev;
	neutrinolum =  neutrinolum + neutrinosppe*surface*ebavg*mev;
	neutrinolum =  neutrinolum + neutrinosppe*surface*enavg*mev;
	neutrinolum =  neutrinolum + neutrinosppe*surface*eoavg*mev;

	fprintf(fp,"L_neutrino       = %e Watts\n",neutrinolum);
	fprintf(fp,"L_neutrino/L_sun = %e = %f percent.\n",neutrinolum/lsun,(neutrinolum/lsun)*100.0);
}
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .