#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
//                                    June 25, 2002.
//  starz6fn.cpp
//               - More Main Menu functions & supporting files.             FILE
//
// void mstars(void);      // - Characteristics of Main Seq. stars (V).  z6mnseq.txt
// void giants(void);      // - Characterisitics of Giants (III).        z6giants.txt
// void supergiants(void); // - Characteristics of Supergiants (I).      z6superg.txt
// void sketchpoly(void);  // - polytropic MR modeler with L_nuclear.    z6sketch.txt
// void astrocalc(void);   // - Astrophysical calculator.                z6astcal.txt
// void bolocal(void);     // - Bolometric calc from measurements.       z6bocal.txt
//
//   EXTERNAL VARIABLES - defined in starz6.cpp.
//
// extern int i;              // model index. Set to 0.
extern int masslumoption; //  Mass-Luminosity Law equation options.
//           0 -> M/Msun = exp[(1.0/3.409)*In(L/Lsun)], from chart measurements, original.
//           1 -> M/Msun = exp[{1.0/beta(L/Lsun)}*In(L/Lsun)],
//                         beta(L/Lsun) = exp[In(L/Lsun)/(0.0525568*log10(L/Lsun) + 3.262926)],
//                         Based on Bowers & Deeming Main Sequence Properties at extremes.
//           2 -> M/Msun = exp[{1.0/beta2(L/Lsun)}*In(L/Lsun)],
//			   beta(L/Lsun) = 3.558592857, Bowers & Deeming Main Seq. avg.
//	     3 -> M/Msun = exp[{1.0/beta(L/Lsun)}*In(L/Lsun)],
//                         beta(L/Lsun) = exp[In(L/Lsun)/(A*log10(L/Lsun) + B)],
//                         Here A & B will be chosen to fit our ZAMS Mass-Luminosity
//                         law based on nuclear astrophysics and stellar struc. theory.
extern int mrlmodeler;      // Set equal to 1 during MR modeler for report titles.
extern int msmodels;       // models to process. Set to 1.
extern double d;		   // integer string entry value
extern double rsun;        // radius of the Sun in meters.
extern double lsun;        // luminosity of the Sun in watts.
extern double sigma;       // blackbody radiation constant.
extern char value[50];     // string for entry value
extern double lightyears[1]; // distance to model star in LY.
extern double visualmag[1];       // visual magnitude of model star i.
extern double bcmratiobest[1];  // M/Msun with bolometric correction, best fit.
extern double bcrratio[1];      // R/Rsun with bolometric correction B.C.
extern void probe(int i);    // Test probe at point i.
extern void elements(void);   // manage the table of the elements.
extern void initspectable(void);  // initialize spectral_type -> surface temperature table.
extern void initpolytrope(void);  // initialize rx radial points, theta(rx) for n=3 polytrope.
extern void processmsmodels(void); // prepare screen report.
extern void reportms(void);        // prepare file report.
extern int starform[1];           // Assumed structure type:
//                                0->n=3, gamma=4/3, Standard Model.
//                                1->n=1.5, gamma=5/3 polytrope.
extern double surfacetemp[1];   // Surface temp in degrees Kelvin.
extern double bclratio[1]; // L/Lsun, bolometrically exact for above atmosphere.
extern void setspectype(double tempsurf, int i); // prototype for lookup
//                                           spec type equiv.
//                        from the surface temperature for i_th entry.
extern double bolometric(double xt); // Look up bolometric correction BC
//                                      from xt = log10(surfacetemp[0]).
struct spectral              // spectral type with temperature.
  {
  char spectcode; // spectral OBAFGKM type.
//                   V specifies the Sun for calibration at 5770 K.
  double spectnumber;  // number within type, such as B3, O2, A4, ect.
  int classification;  // 1=giant (III)  2=regular (V).
  double surftemp;  // surface temperature associated with spectral type,
  int equiv; // =1 if spectral type is deduced from temperature, else 0.
  char starname[50];       // star name.
  };                   //      same as surfacetemp.

extern struct spectral sp[1], *spec_ptr; // spectral type & temp for model i.


//
//   GLOBAL VARIABLES
//
double modeltype = 0;          // 0->MR-modeler L/Lsun determined by mass-luminosity law,
//                                   R/Rsun determined by user specification.
//                                1->MR-modeler L/Lsun is taken from a chart,
//                                   Teff also taken from chart, determining R/Rsun(L,Teff).
double luminositymass(double stmass);  // Find the luminosity ratio L/Lsun from the mass ratio
//                         M/Msun using formulae selected by masslumoption 1-4, above.
double massluminosity(double lumin); // Find the mass ratio M/Msun from the luminosity ratio
//                         L/Lsun using formulae selected by masslumoption 1-4, above.
void mstars(void);         // - Characteristics of Main Sequence stars (V).
void giants(void);         // - Characterisitics of Giants (III).
void supergiants(void);    // - Characteristics of Supergiants (I).
void sketchpoly(void);     // - Polytrope quick sketches.
void astrocalc(void);      // - Astrophysical calculator.
void bolocal(void);        // - Bolometric calculations from measurements
void readpmenu(void);      // - Read polytropic sketch menu.
double pwr(double b, int m);  // return the mth power of b.
double logg(double v);      // log_e(v), custom.
double logg2(double v);      //  subroutine for log_e(v), custom,
double logg4(double v);      //  subroutine for log_e(v), custom.
double series(double b, int nn); // Taylor series for log_e(v).
int status2;
FILE *fp2;	

int fclose(FILE *fp2);
char rdbuf;              //  buffer input one character at a time. Stream ends with $
//-----------------------------------------
//
//  mstars() - Display M.S. star characteristics from file mainseq.txt.
//
//-----------------------------------------
void mstars(void)         // - Display characteristics of Main Sequence stars (V).
{

	int select;
	int maxcntr;

	fp2 = fopen("z6/z6mnseq.txt", "r");
	if(fp2 == NULL)
	   printf("Cannot read file z6/z6mnseq.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp2,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp2);

}
//-----------------------------------------
//
//  giants() - Display giant star characteristics from file giants.txt.
//
//-----------------------------------------
void giants(void)         // - Display characterisitics of Giants (III).
{
	int select;
	int maxcntr;

	fp2 = fopen("z6/z6giants.txt", "r");
	if(fp2 == NULL)
	   printf("Cannot read file z6/z6giants.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp2,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp2);
}
//-----------------------------------------
//
//  supergiants() - Display giant star characteristics from file supergi.txt.
//
//-----------------------------------------
void supergiants(void)    // - Characteristics of Supergiants (I).
{
	int select;
	int maxcntr;

	fp2 = fopen("z6/z6giants.txt", "r");
	if(fp2 == NULL)
	   printf("Cannot read file z6/z6superg.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp2,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}
	

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp2);
}
//-----------------------------------------
//
//  sketchpoly() - Walk the user through entries for MR-modeler.
//
//      out: if modeltype = 0 and mrmodeler = 1,
//              then M/Msun and R/Rsun are specified at entry time,
//              L/Lsun is determined by the the mass-luminosity law,
//              and Teff is determined by Teff(L/Lsun,R/Rsun). Then
//              processmsmodels() and reportms() calculate L_nuclear, ect.
//           if modeltype = 1 and mrmodeler = 1, the user chose the alternative,
//              and then M/Msun is specified, but R/Rsun entry is overridden
//              after the user is prompted to enter L_observed/Lsun and Teff, 
//              so that R/Rsun = F(L_observed/Lsun, Teff/Tsun).
//              Then L_nuclear/Lsun = F2(M,R). We can use this seperate calc.
//              to search for M such that L_nuc = L_observed, then use
//              M and L to find the theoretical mass-luminosity curve for
//              the ZAMS main sequence.
//
//-----------------------------------------
void sketchpoly(void)     // - Polytrope quick sketches.
{
	int select;	
	int okflag3;	
	double dlyears;   // distance to the model star in LY.
	double tempsurf;  // surface temp of the model star
	double xt, xtsun; // log10(tempsurf), log10(tempsurf_Sun).
	double q1,vi;
	double lumratio;  // Save bclratio[0]
	double savemratio;
	double saverratio;
	int saveform;      // save starform, polytropic type spec.
	double g;	
	int gk;
	int select2;

	okflag3 = 1;
	while(okflag3 == 1){
	   readpmenu();

           printf("ENTER the Mass in solar masses: ");
	   scanf("%s",value);       // Entry procedure more immune to
	   g = atof(value);         // invalid inputs than simple scanf
	   bcmratiobest[0] = g;
	   savemratio = bcmratiobest[0];
	   readpmenu();

	   printf("M/Msun = %f.\n", bcmratiobest[0] );
	   printf("ENTER 0 for stnd model n=3 polytrope, else 1 for n=1.5: ");
	   scanf("%s",value);       // Entry procedure more immune to
	   d = atof(value);         // invalid inputs than simple scanf
	   saveform = d;            // save polytropic starform spec

	   if(saveform != 0 && saveform != 0)
		saveform = 0;
	   starform[0] = saveform;

	   readpmenu();

	   printf("M/Msun = %f, starform = %d.\n", bcmratiobest[0], starform[0]);
	   printf("The radius R may be specified directly. Then  L/Lsun is computed\n");
	   printf("from M/Msun and Teff is computed from Teff(L/Lsun,R/Rsun).\n");
	   printf("Alternatively R/Rsun is computed as R(L/Lsun,Teff),\n");
	   printf("where L/Lsun & Teff are observed and L_nuclear(R,M) is computed.\n");
	   printf("ENTER 0 to specify R, or 1 to determine R(L/Lsun,Tsurface): ");
	   scanf("%s",value);       // Entry procedure more immune to
	   d = atof(value);         // invalid inputs than simple scanf
	   select = d;
	   modeltype = select;
	   if(modeltype != 0 && modeltype != 1)
	      modeltype = 0;     // if modeltype not 0 and not 1, force 0.

	   if(modeltype == 0){
	     printf("ENTER the Radius in solar radii: ");
	     scanf("%s",value);       // Entry procedure more immune to
	     g = atof(value);         // invalid inputs than simple scanf
	     bcrratio[0] = g;

	     readpmenu();

   
	     bclratio[0] = luminositymass(bcmratiobest[0]);  // new luminosity(mass) formulae selection
	     q1 = bclratio[0]/(bcrratio[0]*bcrratio[0]);  // Set the global variable for
//                                                   bolometrically correct L/Lsun.
	     q1 = sqrt(q1);
	     q1 = sqrt(q1);
	     surfacetemp[0] = 5770.0*q1;
  
	   
	     printf("M/Msun = %f, R/Rsun = %f\n", bcmratiobest[0],bcrratio[0]);
	     printf("Option %d Mass-Luminosity Law L/Lsun = %f. \n",masslumoption, bclratio[0]);
	     printf("Surface Temperature(R,L(M)) = %f degrees Kelvin.\n",surfacetemp[0]);
	   }	   
	   else if(modeltype == 1){      // Set up to specify Lobserved/Lsun independantly
	     printf("M/Msun = %f.\n", bcmratiobest[0] );
	     printf("ENTER Lobserved/Lsun: ");
	     scanf("%s",value);       // Entry procedure more immune to
	     d = atof(value);
	     bclratio[0] = d;
	     printf("L_observed/Lsun = %f. \n", bclratio[0]);
	     printf("ENTER Surface temperature: ");
	     scanf("%s",value);       // Entry procedure more immune to
	     d = atof(value);
	     surfacetemp[0] = d;
	     bcrratio[0] = (5770.0/surfacetemp[0])*(5770.0/surfacetemp[0])*sqrt(bclratio[0]);
	     printf("Surface Temperature = %f degrees Kelvin.\n",surfacetemp[0]);
	     printf("L_observed/Lsun = %f. \n", bclratio[0]);
	     printf("Trial M/Msun = %f, R/Rsun(L_observed,Teff) = %f.\n",
	       bcmratiobest[0], bcrratio[0]);
	     printf("ENTER 1 to continue with values above: ");
	     scanf("%s",value);       // Entry procedure more immune to
	     d = atof(value);	     
	   }   

	   saverratio = bcrratio[0];

	   printf("ENTER 1 to examine or modify the chemistry: ");
	   scanf("%s",value);       // Entry procedure more immune to
	   d = atof(value);         // invalid inputs than simple scanf
	   select = d;
	   
	   elements();

	   readpmenu();

	   printf("Surface Temperature = %f degrees Kelvin.\n",surfacetemp[0]);
	   printf("M/Msun = %f, R/Rsun = %f, L/Lsun = %f. \n",
	      bcmratiobest[0],bcrratio[0],bclratio[0]);
	   printf("ENTER Star_Name, no spaces, < 50 chars: ");
	   scanf("%s",sp[0].starname);

	   readpmenu();

	   printf("Surface Temperature = %f degrees Kelvin.\n",surfacetemp[0]);
	   printf("M/Msun = %f, R/Rsun = %f, L/Lsun = %f. \n",
	     bcmratiobest[0],bcrratio[0],bclratio[0]);
	   printf("StarName: %s.\n",sp[0].starname);
	   printf("Solar distance is 1.581395349e-5 LY.\n");
	   printf("ENTER the model distance in Light-Years: ");
	   scanf("%s",value);       // Entry procedure more immune to
	   lightyears[0] = atof(value);         // invalid inputs than simple scanf

	   readpmenu();

	   printf("Surface Temperature = %f degrees Kelvin.\n",surfacetemp[0]);
	   printf("M/Msun = %f, R/Rsun = %f, L/Lsun = %f. \n",
	     bcmratiobest[0],bcrratio[0],bclratio[0]);
	   lumratio = bclratio[0];  // save to restore: blcratio[0] = lumratio;
	   printf("StarName = %s, Distance = %f LY.\n", sp[0].starname,lightyears[0]);
	   printf("Class codes   :   1->I,Ia,supergiants, 2->II,bright giants\n");
	   printf("3->III, giants,   4->IV, subgiants,    5->V, ZAMS Main Seq.\n");
	   printf("6->VI, subdwarfs, 7->VII,white dwarfs, 8->V,dwarf, or unknown.\n");
	   printf("ENTER preliminary classification code 1-8, or 0 to abort: ");
	   scanf("%s",value);       // Entry procedure more immune to
	   d = atof(value);         // invalid inputs than simple scanf
	   select = d;
	   select2 = d;             // Rewrite classification as required.
	   if(select == 3)          // Remap classification to internal code from user code
	      select2 = 1;          //  giant (III)    -> 1 from 3
	   else if(select == 1)     //  dwarf (V)      -> 8 from 8  no change
	      select2 = 3;          //  supergiant (I) -> 3 from 1
	   else if(select == 8)     //  subgiant (IV)  -> 4 from 4  no change
	      select2 = 2;          //  ZAMS (V)       -> 5 from 5  no change
           else if(select > 8)      //  Subdwarfs (VI) -> 6 from 6  no change
	    select2 = 8;	    //  Wht dwfs (VII) -> 7 from 7  no change
//                                      BriteGiant (II)-> 2 from 8
	   sp[0].classification = select2;

	   if(select !=  0){
	       printf("\nSTARCAL\n");
	       mrlmodeler = 1;     // Set mrlmodeler -> 1 for MRL titles on reports.
	       msmodels = 1;       // Declare one model to process

	       printf("Surface Temperature = %f degrees Kelvin.\n",surfacetemp[0]);
	       printf("M/Msun = %f, R/Rsun = %f, L/Lsun = %f. \n",
		  bcmratiobest[0],bcrratio[0],bclratio[0]);
	       printf("StarName = %s, Distance = %f LY.\n", sp[0].starname,lightyears[0]);
	       initspectable();  // initialize spectral_type -> surface temperature table.
	       initpolytrope();  // initialize rx radial points, theta(rx) for n=3 polytrope.
//                                  set up exterior distance variable measures
	       starform[0] = saveform; // Assumed structure type:
//                                        0->n=3, gamma=4/3, Standard Model.
//                                        1->n=1.5, gamma=5/3 polytrope
	       printf("n=3, gamma=4/3 polytrope, H-depleted core: Main Sequence assumed.\n");
               
//                       Compute surface temp from L  = 4*pi*(R**2)*sigma*(Teff**4).
//                            But        Lsun  = 4*pi*(Rsun**2)*sigma*(Teff_sun**4).
//                           Thus  L/Lsun  =  ((R/Rsun)**2)*(Teff/Teff_sun)**4,
//                             so   Teff/Teff_sun =  [(L/Lsun)*((Rsun/R)**2)]**(1/4).  
  
    

	       sp[0].surftemp = surfacetemp[0];
	       sp[0].equiv = 1;  // spectral type equivilant
//                                  must be deduced from surf. temp.
	       setspectype(surfacetemp[0],0); // set the spectral type from the surface temp.

	       
//                       Compute the visual magnitude of the star.
//                             I/Isun =  10**(2/5)*(m_sun_bol - m_bol)
//                             log(I/Isun) =  (2/5)*(m_sun_bol - m_bol)*log(10.0)
//                             m_sun_bol - m_bol = log(I/Isun)/( (2/5)*log(10) )
//                             m_bol = m_sun_bol - log(I/Isun)/( (2/5)*log(10) ),
//
//                       but   m_sun_bol = m_sun + BC = -26.74 + BC(Teff_sun)
//                       and   m_bol = m_vis + BC(Teff),
//
//   so    m_vis + BC(Teff) =  -26.74 + BC(Teff_sun) - log(I/Isun)/((2/5)*log(10)),
//
//   or    m_vis  =  -26.74 + BC(Teff_sun) - BC(Teff) - log(I/Isun)/((2.0/5.0)*log(10.0)),
//
//       But I/Isun = (L/(4*pi*d**2))*((4*pi*R_es**2)/Lsun) = (L/Lsun)*(Res/d)**2

	       bclratio[0] = lumratio;  //  restore  blcratio[0] = lumratio;
	       bcmratiobest[0] = savemratio;
	       bcrratio[0] = saverratio;

	       q1 = (bclratio[0])*(1.496e11/(9.46e15*lightyears[0]));
	       q1 = q1*(1.496e11/(9.46e15*lightyears[0]));

	       vi =  -26.74 - log(q1)/((2.0/5.0)*log(10.0));

	       xtsun = log10(5770.0);
	       xt = log10(surfacetemp[0]);
	       visualmag[0] = vi + (bolometric(xtsun) - bolometric(xt));
	       printf("At %e LY, m_vis = %f, ",lightyears[0], visualmag[0]);

	       g = sp[0].spectnumber;
	       if(g == 0.0 || g == 1.0 || g == 2.0 || g == 3.0
	       || g == 4.0 || g == 5.0 || g == 6.0 || g == 7.0
	       || g == 8.0 || g == 9.0 || g == 10.0){
	         gk = g; // subsitute integrer for fractional type.
		 printf("Spectral type %c%d.\n", sp[0].spectcode, gk);
	       }

	       processmsmodels();  // process the list of main sequence models.
	       reportms();   // prepare starexp.txt report file from mem.vars.
	       printf("Report=starmodl.txt,STARMODL directory. ");
	       printf("ENTER 0->menu,1->MR Modeler:");
	       scanf("%s",value);       // Entry procedure more immune to
	       d = atof(value);         // invalid inputs than simple scanf
	       select = d;
	   }
	   if(select ==  0)  //  EXIT request from polytrope sketch.
	      okflag3 = 0;

	}
}
//
//  readpmenu() - Read the menu file for sketchpoly()
//
void readpmenu(void)
{
	   int maxcntr;

           fp2 = fopen("z6/z6sketch.txt", "r");
	   if(fp2 == NULL)
	      printf("Cannot read file z6/z6sketch.txt.\n");
           else{
	      maxcntr = 0;
	      rdbuf = 0;
	      while(rdbuf  != '$' && maxcntr < 2000){
	        fscanf(fp2,"%c", &rdbuf);
		if(rdbuf != '$')
	          printf("%c", rdbuf);
		maxcntr++;
	      }
	   }
	   fclose(fp2);
}
//------------------------------------------------------------
//
//   luminositymass(double massratio) - cal luminosity ratio from mass ratio.
//
//   masslumoption = 0; //  Mass-Luminosity Law equation options.
//           0 -> L/Lsun = exp[(3.409)*In(M/Msun)], from chart measurements, original.
//           1 -> L/Lsun = exp[{betax(M/Msun)}*In(M/Msun)],
//                         find betax.
//                         Based on Bowers & Deeming Main Sequence Properties at extremes.
//           2 -> L/Lsun = exp[beta2*In(M/Msun)],
//			   beta2(L/Lsun) = 3.558592857, Bowers & Deeming Main Seq. avg.
//	     3 -> L/Lsun = exp[{1.0/betax(M/Msun)}*In(L/Lsun)],
//                         Find betax(M/Msun)
//                         Here A & B will be chosen to fit our ZAMS Mass-Luminosity
//                         law based on nuclear astrophysics and stellar struc. theory.
//
//------------------------------------------------------------
double luminositymass(double stmass)
{
       double beta;       
       double aa2 = 0.4625;      // follows option 1, to be defined for custom main seq. 
       double bb2 = 3.277884615; // analysis to be defined in option 3.

       if(masslumoption == 0)
	   return(exp(3.409*log(stmass)));
       else if(masslumoption == 1){
	      beta = 0.4625*log10(stmass) + 3.277884615;
	      return(exp((beta)*log(stmass)));
       }
       else if(masslumoption == 2){
	      beta = 3.558592857;
	      return(exp(beta*log(stmass)));
       }
       else if(masslumoption == 3){
	      beta = aa2*log10(stmass) + bb2;
	      return(exp((beta)*log(stmass)));
       }
       else{
	   masslumoption = 0;
	   return(exp(3.409*log(stmass))); // default for code is original.
       }
}
//--------------------------------------------------------
//
//    massluminosity(double lumin) - Call mass ratio from luminosity ratio.
//
//   masslumoption = 0; //  Mass-Luminosity Law equation options.
//           0 -> M/Msun = exp[(1.0/3.409)*In(L/Lsun)], from chart measurements, original.
//           1 -> M/Msun = exp[{1.0/beta1(L/Lsun)}*In(L/Lsun)],
//                         beta1(L/Lsun) = 0.0525568*log10(L/Lsun) + 3.262926,
//                         Based on Bowers & Deeming Main Sequence Properties at extremes.
//           2 -> M/Msun = exp[{1.0/beta2(L/Lsun)}*In(L/Lsun)],
//			   beta2(L/Lsun) = 3.558592857, Bowers & Deeming Main Seq. avg.
//	     3 -> M/Msun = exp[{1.0/beta(L/Lsun)}*In(L/Lsun)],
//                         beta(L/Lsun) = A*log10(L/Lsun) + B,
//                         Here A & B will be chosen to fit our ZAMS Mass-Luminosity
//                         law based on nuclear astrophysics and stellar struc. theory.
//
//----------------------------------------------------------
double massluminosity(double lumin)
{
       double beta;
       double aa = 0.0525568; // for option 3 to be defined for custom main seq. analysis
       double bb = 3.262926; //  presently set to follow option 1.

       if(masslumoption == 0)
	      return(exp((1.0/3.409)*log(lumin)));
       else if(masslumoption == 1){
	      beta = 0.0525568*log10(lumin) + 3.262926;
	      return(exp((1.0/beta)*log(lumin)));
       }
       else if(masslumoption == 2){
	      beta = 3.558592857;
	      return(exp((1.0/beta)*log(lumin)));
       }
       else if(masslumoption == 3){
	      beta = (aa*log10(lumin) + bb);
	      return(exp((1.0/beta)*log(lumin)));
       }
       else{
	      masslumoption = 0;
	      return(exp((1.0/3.409)*log(lumin))); // default for code is original.
       }
}
//
// astrocalc() - Astrophysical calculator.
//
void astrocalc(void)      
{
	int select;
	int maxcntr;
	int okflag3;

	okflag3 = 1;
	while(okflag3 == 1){
	   fp2 = fopen("z6/z6astcal.txt", "r");
	   if(fp2 == NULL)
	      printf("Cannot read file z6/z6astcal.txt.\n");
           else{
	      maxcntr = 0;
	      rdbuf = 0;
              while(rdbuf  != '$' && maxcntr < 2000){
	        fscanf(fp2,"%c", &rdbuf);
	        if(rdbuf != '$')
	          printf("%c", rdbuf);
	        maxcntr++;
	      }
	   }

           printf("ENTER selection: ");
	   scanf("%s",value);       // Entry procedure more immune to
	   d = atof(value);         // invalid inputs than simple scanf
	   select = d;

	   if(select ==  20)  //  EXIT request from astrophysical calculator.
	      okflag3 = 0;

	}

	fclose(fp2);
}
//------------------------------------------------
//
//  bolocal0() - bolometric constants from measurements at different angles.
//
//-------------------------------------------------
void bolocal0(void)
{

	int select;
	int maxcntr;
	int okflag3;

	okflag3 = 1;
	while(okflag3 == 1){
	   fp2 = fopen("z6/z6bocal.txt", "r");
	   if(fp2 == NULL)
	      printf("Cannot read file z6/z6bocal.txt.\n");
           else{
	      maxcntr = 0;
	      rdbuf = 0;
              while(rdbuf  != '$' && maxcntr < 2000){
	        fscanf(fp2,"%c", &rdbuf);
	        if(rdbuf != '$')
	          printf("%c", rdbuf);
	        maxcntr++;
	      }
	   }

           printf("ENTER selection: ");
	   scanf("%s",value);       // Entry procedure more immune to
	   d = atof(value);         // invalid inputs than simple scanf
	   select = d;

	   if(select ==  5)  //  EXIT request from bolometric calculations.
	      okflag3 = 0;

	}

	fclose(fp2);
	
}
//
//      bolometr.cpp - Compute
//      		1) The bolometric correction B.C. for a star
//                         from the measured visual magnitudes m1,m2 at
//			   two zenith angles alpha1 and alpha2.
//                     2) The observed visual magnitude m0 to be expected
//                        at the earths surface for a star at the zenith
//			  given the measured visual magnitudes at the
//			  two zenith angles alpha1 and alpha2.
//   			  This is the star chart magnitude that is
//			  input to STARCAL or STAREXP.
//                     3) The bolometric visual magnitude m_bol
//                        for the star as observed above the atmosphere.
// 
//         The measurement theory is taken from pages 385-386
//	   of THERMONUCLEAR FUSION IN STARS by James A. Green,
//         from the Supplemental Notes for the 1st edition.
//
//                      - Author: James A. Green.
//			- Compiler: Borland C/C++
//
//
void bolocal(void)
{
	char value[20];   // variable input string.
	double d;         // the floating point value of the above string.
	double m1;        // the visual magnitude m1.
	double m2;        // the visual magnitude m2.
	double alpha1;      // the zenith angle alpha1 for m1 in degrees.	
	double alpha2;   // the zenith angle alpha2 for m2 in degrees.
	double alpha1rad;  // the zenith angle alpha1 in radians.
	double alpha2rad;  // the zenith angle alpha2 in radians.
	double sec(double a); // 1.0/cos(a).
	double sec1;    // secant(alpha1).
	double sec2;    // secant(alpha2).
	double bc;      // bolometric correction B.C.
	double m0;	// visual magnitude expected to be observed at zenith angle 0.
	double mbol;    // visual magnitude expected above the atmosphere at zenith angle 0.

	int more = 1;

     while(more == 1){
	printf("bolocal() - Computes:\n");
	printf("  1) The bolometric correction B.C. from observations of\n");
	printf("     the visual magnitudes m1, m2 made at the zenith \n");
	printf("     angles alpha1, alpha2 degrees.\n");
	printf("  2) The visual magnitude m0 expectation value for the\n");
	printf("     star when at the zenith, when alpha is zero.\n");
	printf("     This is the visual magnitude input to STARCAL.\n");
	printf("  3) The corrected magnitude m_bol expected when the\n");
	printf("     visual magnitude is measured above the atmosphere.\n\n");  
	printf("Enter alpha1, the zenith angle for m1 in floating-point: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
	alpha1 = d;          // than simple scanf().

	printf("Enter alpha2, the zenith angle for m2 in floating-point: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
	alpha2 = d;          // than simple scanf().

	printf("Enter m1, the visual magnitude at alpha1 in floating-point: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
	m1 = d;          // than simple scanf().

	printf("Enter m2, the visual magnitude at alpha2 in floating-point: ");
	scanf("%s",value);   // This
	d = atof(value);     // entry procedure is more immune to invalid inputs
	m2 = d;              // than simple scanf().
//
//    Compute results.
//
	printf("\n");
	alpha1rad = (3.14159/180.0)*alpha1;  // convert to radian measure
	alpha2rad = (3.14159/180.0)*alpha2;
	printf("m1 = %f, alpha1 = %f degrees, alpha1 = %f radians\n", m1, alpha1, alpha1rad);
	printf("m2 = %f, alpha2 = %f degrees, alpha1 = %f radians\n", m2, alpha2, alpha2rad);
	

	sec1 = sec(alpha1rad);
	sec2 = sec(alpha2rad);

	printf("\ncos(alpha1) = %f, sec(alpha1)= %f .\n", cos(alpha1rad), sec(alpha1rad));
	printf("cos(alpha2) = %f, sec(alpha2)= %f .\n", cos(alpha2rad), sec(alpha2rad));

	bc = (m2 - m1)/(sec1 - sec2);
	m0 = m1 + bc*(sec1 - 1.0);
	mbol = m0 + bc;

	printf("\n");
	printf("B.C. = %f, m0 = %f, m_bol = %f.\n", bc, m0, mbol);
	printf("REF: Sup.Notes,1st edition,THERMONUC.FUSION IN STARS,pp.385-386.\n");

//
//    Ask for repeat or discontinue decision.
//
	printf("\nEnter 1 to continue or 0 to exit:");
	scanf("%d", &more);
	printf("\n");
	if(more == 0) exit;
     }
}
//
// secant(a) = 1/cos(a)
//
double sec(double a)
{
	double secant;

	secant = 1.0/cos(a);

	
        return(secant);
}
