#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
//
//   starz6fu.cpp - Basic functions for starz6.cpp
//
//
//    - PROJECT STARMAN -  Programmer: James A. Green.
//	   starman.prj is the project file
//	       producing starman.exe from
//		     starz6.cpp - main module.
//		      starz6ag.cpp - Stellar ages and evolution, perturbations.
//		      starz6au.cpp - Neutrino luminosity & stellar wind.
//		      starz6fu.cpp - basic functions.
//		      starz6bo.cpp - bolometric corrections.
//		      starz6op.cpp - supplemental ops.
//
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .
//
//                 
//
//    GLOBAL VARIABLES
//
double findbeta(double fbetanumber);  // solve fit for beta
double nthpower(double x, int n);     // find the nth power of x.
double nthroot(double xx, int nn);    // find the nnth root of xx.
double cuberoot(double v);            // find the cube root of v.
double seventhroot(double v);         // find the seventh root of v.
void probe(int i);		      // halt for the ith probe & wait for keystroke.
//
//    EXTERNAL VARIABLES
//
extern char value[50];   // entry string for value.
extern int d;		 // value of entry string.
/************************************************************
*
*  findbeta - solve (1 - b)/b**4 = C < 1 for ( 0 <= b <= 1 ).
*             Note that since this function wiggles in the
*             interval and is not monotone increasing or
*             decreasing, we have to scan the interval
*             for the best fit.
*
************************************************************/
double findbeta(double fbetanumber)
{
     int cnt;				// counter.
     double fbeta;  // (1-b)/b**4 = fbetanumber when b is known.     
     double b;   // beta.
     double bbest; // best beta in scan of interval
     double epsilon;
     double deltabest;
     double delta;

     epsilon = 1.0/30000.0;  // use 30,000 steps.     
     cnt = 0;
     b = 0.0;
     deltabest = 1.0e8;
     while(cnt < 30000){
        b = b + epsilon;
	fbeta = (1.0 - b)/(b*b*b*b);
	delta = sqrt((fbetanumber - fbeta)*(fbetanumber - fbeta));
	if(delta < deltabest){
	    deltabest = delta;
	    bbest = b;
        }
        cnt++;
     }
     return(bbest);
}
/*****************************************
*
*   nthpower(x,n) - Return x**n.
*
*   n is an integer which may be less than or equal to zero.
*
*******************************************/
double nthpower(double x, int n)
{
	 double v;
	 int i;
         int m;

	 if(n == 0){     // n == 0?
	   v = 1.0;
	   return(v);
	 }
	 if(n < 0){       //  n < 0?
	    m = -n;
	    v = 1.0;
	    i = 0;
	    while(i < m){
	       v = v*x;
	       i++;
	    }
	    v = 1/v;
            return(v);
	 }
//
// well, then n > 0, so that we use
//
	 v = 1.0;
	 i = 0;
	 while(i < n){
            v = v*x;
            i++;
	 }
         return(v);
}
/******************************************
*
*  nthroot(x,n) - Cal the nth root of x, x**(1/n).
*
*
*******************************************/
double nthroot(double xx, int nn)
{
      int vi, i, doitagain;
      double vdelta,diff,testval,midpnt,lower,upper;
      double xu,epsilon,sign;

      epsilon = 0.00000001;
      xu = xx;
      sign = 1.0;
      if( xu < 0.0){
	 sign = -1.0;
	 xu = -xu;
      }
      lower = 0.0;
      if(xu >= 1.0)
	  upper = 1.5*sqrt(xu); // use tagged sqrt().
      else
	  upper = 1.0;
      doitagain = 1;

      i = 0;                           /* binary search loop */
      while( i < 1000 && doitagain == 1){
	 midpnt = lower + ((upper - lower)/2.0);
	 testval = 1.0;
	 for(vi = 0; vi < nn; vi ++)
	    testval = midpnt*testval;
	 diff = testval - xu;
	 vdelta = diff;
	 if( vdelta < 0.0)
	    vdelta =  0.0 - vdelta;
	 if( vdelta > epsilon)
	    doitagain = 1;
	 if( vdelta <= epsilon)
	    doitagain = 0;
	 if( diff < 0.0 )
	    lower = midpnt;
	 if( diff > 0.0 )
	    upper = midpnt;
	 if( diff = 0.0 )
	    doitagain = 0;
	 i = i + 1;
      }
      return(sign*midpnt);
}
/*******************************************
*
*   cuberoot(v) - Compute the cube root of v.
*
********************************************/
double cuberoot(double v)
{
      double zout;
      double sign;

      if(v < 0.0){
	sign = -1.0;
        v = -v;
      }
      else
	sign = +1.0;
      
      zout = nthroot(v,3);
      zout = zout*sign;
      return(zout);
}
/********************************************
*
*   seventhroot(v) - Compute the seventh root of v.
*
*
*********************************************/
double seventhroot(double v)
{
       double zout;

       zout = nthroot(v,7);
       return(zout);
}
/****************************
*
*   probe(i) - Hit any key to continue from stop i.
*              This is how the probe identifies itself as the
*              ith probe.
*
****************************/
void probe(int i)
{
      int number;

      printf("Enter any number to continue from probe %d :\n",i);
      scanf("%s",value);       // Entry procedure more immune to
      d = atof(value);         // invalid inputs than simple scanf
      number = d;
}
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .