//
//  starz4ops.cpp - Module for STARZ4 - Supplemental Ops
//
//                  Supporting text files:
//                  z6tsurf.txt, z6spect.txt, z6listm.txt, z6brite.txt,
//                  z6near.txt, z6help.txt, z6dwarf.txt.
//
//
//    - PROJECT STARMAN -  Programmer: James A. Green.
//	   starman.prj is the project file
//	       producing starman.exe from
//		     starz4.cpp - main module.
//		      starz4ag.cpp - Stellar ages and evolution, perturbations.
//		      starz4au.cpp - Neutrino luminosity & stellar wind.
//		      starz4fu.cpp - basic functions.
//		      starz4bo.cpp - bolometric corrections.
//		      starz4op.cpp - supplemental ops.
//
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .
//
//
//       CONTENTS
//
//       void tsurf(void) - View Tsurface from spectral type.
//       void spectraldata(void) - Display reference spectral data.
//       void listmainseq(void) - List properties of main seq. stars.
//       void brighteststars(void)  - The brightest stars, properties.
//       void neareststars(void) -  The nearest stars, properties.
//       void morehelp(void)  - More help - references.
//       void whitedwarfs(void)     - Characteristics of White Dwarfs
//
//     EXTERNAL LIBRARIES
//
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
//
//    GLOBAL VARIABLES - defined here in starz4op.cpp & globally known.
//
//				PROTOTYPE DEFINITIONS
void tsurf(void);          // - View Tsurface from spectral type.
void spectraldata(void);   // - Display reference spectral data.
void listmainseq(void);    // - List properties of main seq. stars.
void brighteststars(void); // - The brightest stars, properties.
void neareststars(void);   // -  The nearest stars, properties.
void morehelp(void);       // - More help - references.
void whitedwarfs(void);    // - Characteristics of White Dwarfs
int status4;
FILE *fp4;	

int fclose(FILE *fp4);

//
//   EXTERNAL VARIABLES - defined in starz6.cpp.
//
extern int d;		   // integer string entry value
extern char value[50];     // string for entry value

extern char rdbuf;         // char buffer defined in starz6op.cpp
/**************************************************************
*
*  tsurf() - View Tsurface from the spectral type.
*            text file: z6tsurf.txt.
*
***************************************************************/
void tsurf(void)
{
        int proceedflag = 0;

        int select;
	int maxcntr;

	fp4 = fopen("z6/z6tsurf.txt", "r");
	if(fp4 == NULL)
	   printf("Cannot read file z6/z6tsurf.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp4,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp4);
        return;
}
/***********************************************
*
*  spectraldata() - Display spectral data - Reference
*                   text file: z6spect.txt.
*
************************************************/
void spectraldata(void)
{
        int select;
	int maxcntr;

	fp4 = fopen("z6/z6spect.txt", "r");
	if(fp4 == NULL)
	   printf("Cannot read file z6/z6spect.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp4,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp4);
}
/**************************************************************
*
*   listmainseq() - List properties of well-known Main Sequence
*                   stars for use in polytrope() sketch routine
*                   above.
* 		    text file: z6listm.txt.
*
****************************************************************/
void listmainseq(void)
{
	int select;
	int maxcntr;

	fp4 = fopen("z6/z6listm.txt", "r");
	if(fp4 == NULL)
	   printf("Cannot read file z6/z6listm.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp4,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp4);
}
/*****************************************
*
*   brighteststars() - display characteristics of brightest stars
*                      for calculational input purposes.
*                      text file: z6brite.txt.
*
******************************************/
void brighteststars(void)
{
        int select;
	int maxcntr;

	fp4 = fopen("z6/z6brite.txt", "r");
	if(fp4 == NULL)
	   printf("Cannot read file z6/z6brite.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp4,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp4);

     
}
/*****************************************
*
*   neareststars() - display characteristics of nearest stars
*                      for calculational input purposes.
*                      text file: z6near.txt.
*
******************************************/
void neareststars(void)
{
	int select;
	int maxcntr;

	fp4 = fopen("z6/z6near.txt", "r");
	if(fp4 == NULL)
	   printf("Cannot read file z6/z6near.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp4,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp4);
}
/**********************
*
*  morehelp -  Supply references.
*              text file: z6help.txt.
*
**********************/
void morehelp(void)
{
        int select;
	int maxcntr;

	fp4 = fopen("z6/z6help.txt", "r");
	if(fp4 == NULL)
	   printf("Cannot read file z6/z6help.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp4,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp4);
}
/*****************************************
*
*   whitedwarfs() - display characteristics of white dwarfs
*                      for calculational input purposes.
*                   Much of the table data below is culled
*                   from BURNHAMS CELESTIAL HANDBOOK, pg.400-434
*
*                   text file: z6dwarf.txt
******************************************/
void whitedwarfs(void)
{
	int select;
	int maxcntr;

	fp4 = fopen("z6/z6dwarf.txt", "r");
	if(fp4 == NULL)
	   printf("Cannot read file z6/z6dwarf.txt.\n");
	else{
	   maxcntr = 0;
	   rdbuf = 0;
           while(rdbuf  != '$' && maxcntr < 2000){
	     fscanf(fp4,"%c", &rdbuf);
	     if(rdbuf != '$')
	       printf("%c", rdbuf);
	     maxcntr++;
	   }
	}

        printf("ENTER any number to return to the main menu: ");
        scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	fclose(fp4);

}

/*
*     References
*
* [1] - Richard Bowers & Terry Deeming, 1984. Astrophysics I: The Stars.
*       Jones & Barlett.
* [2] - Hong Yee Chiu, STELLAR PHYSICS, Ginn Blaisdell.
* [3] - Stuart L. Shapiro & Saul A. Teukolsky, 1983.
*       Black Holes, White Dwarfs, and Neutron Stars. Wiley-Interscience.
* [4] - Green, James A., CHANDRA: Chandrasekhar Sets of White Dwarfs
*           and Supernova Cores with Stellar Structure Software, 1997.
* [5] - Green, James A., THERMONUCLEAR FUSION IN STARS, Greenwood
*          Research Books & Software, 1999.
*       Shows pre-supernova stars have M/Msun > 6.23.
* [6] - A.C.Phillips, The Physics of Stars, John Wiley & Sons,pg.142-146.
* [7] - Donald D. Clayton, Principles of Stellar Evolution &
*       Nucleosynthesis, McGraw-Hill.
* [8] - Martin Schwarzschild, Structure & Evolution of the Stars,
*        Dover Pulbications, pg.9, bolometric corrections.
* [9] - Martin V. Zombeck, Handbook of Space Astronomy & Astrophysics,
*       Cambridge University Press, pp.68-69 for bolometric corrections.
* [10] - William K. Rose, Advanced Stellar Astrophysics,
*        Cambridge University Press.
* [11] - Martin V. Schwarzschild, STRUCTURE & EVOLUTION
*                                 OF THE STARS, Dover.
* [12] - E.E.Salpeter, "Nuclear Reactions in the Stars. I.
*        Proton-Proton Chain",
*        Physical Review, Volume 88, Number 3, Nov.1, 1952.
* [13] - H.A.Bethe and C.L.Critchfield, "The Formation of
*        Deuterons by Proton Combination", August 15, 1938,
*        Physical Review, pp.248-254.
*
*/
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .